/*
Chip type           : AT90S2313
Clock frequency     : 2,000000 MHz
Memory model        : Tiny
Internal SRAM size  : 128
External SRAM size  : 0
Data Stack size     : 32
*********************************************/

#include <90s2313.h>
                        
#define offen 1
#define geschlossen 2
#define unbekannt 3                                
#define statusadresse 10

#define f_auf 20
#define f_zu 30
#define f_non 40
  
// b7, b6 = Fernbedienung !

// Declare your global variables here
int z1,z2,z3;      
char status;			// Stauts der Schlsser
char speicherstatus;            // Status im EEPROM
char altstatus;

// Fernbedienung reinholen
char get_fernbedienung (void)
 {                        
  if (PINB & 128) return (f_auf);
  if (PINB & 64) return (f_zu);
  return (f_non);
 }        

// Zustand erkennen
char get_zustand (void)
 {
  // char temp;
  
  // Fernbedinung hat immer Vorrang
  if (get_fernbedienung () == f_auf) return (offen);
  if (get_fernbedienung () == f_zu) return (geschlossen);
  
  if ((PIND & 16) && (PIND & 32)) return (unbekannt); else // Beide 1
   if (!(PIND & 16) && !(PIND & 32)) return (unbekannt); else // Beide 0
 
  if (PIND & 16) return (offen); else // Schlsser offen
  if (PIND & 32) return (geschlossen); // Schlsser dicht
 }
             
//EEPROM  write
void write_eeprom (char address, char value)
{
   while (EECR & 0x02);       //wait for EEWE==0 (EECR.1)
   EEAR = address;            //write address to EEAR
   EEDR = value;              //wrie data to EEDR
   EECR = 0x04;               //set EEMWE=1 (EECR.2)
   EECR = 0x02;               //set EEWE=1 (EECR.1)
}//end function


//EEPROM  read, returns data byte       
char read_eeprom (char address)
{  
   while (EECR & 0x02);       //wait for EEWE==0
   EEAR = address;            //write address to (EEAR)
   EECR = 0x01;               //set EERE=1 (EECR.0)
   return (EEDR);
}//end function
           
// kleine Pause einlegen
void delay (void)
 {          
  for (z1=1;z1<3;z1++)
   for (z2=1;z2<8000;z2++)
    {
	#asm("wdr");    
    	z3=z1+z2;
    }	
 }
 
 // ganz kleine Pause einlegen
void sdelay (void)
 {          
   for (z2=1;z2<8000;z2++)
    {
	#asm("wdr");    
    	z3=z1+z2;
    }	
 }
                      
// Schlsser ffnen
void auf (void)
 {                 
  PORTB|=2;    
  delay ();
  PORTB&=255-2;          
  write_eeprom (statusadresse,offen);
  status=offen;
  speicherstatus=offen;
 }

// Schlsser schliessen
void zu (void)
 {                 
  PORTB|=1;    
  delay ();
  PORTB&=255-1;
  write_eeprom (statusadresse,geschlossen);
  status=geschlossen;
  speicherstatus=geschlossen;
 }


void main(void)
{
// Declare your local variables here

// Input/Output Ports initialization
// Port B
DDRB=0x3f;  // Alles auf Ausgang, B6, B7 = Eingang
PORTB=0x00;

// Port D
DDRD=0x00;
PORTD=0xff;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: Timer 0 Stopped
// Mode: Output Compare
// OC0 output: Disconnected
TCCR0=0x00;
TCNT0=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer 1 Stopped
// Mode: Output Compare
// OC1 output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
OCR1H=0x00;
OCR1L=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
GIMSK=0x00;
MCUCR=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x00;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;

// Watchdog Timer initialization
// Watchdog Timer Prescaler: OSC/2048
WDTCR=0x0F; 

status=get_zustand ();       
speicherstatus=read_eeprom (statusadresse);

if (status==unbekannt)
 {                            // Zustand ist Quark, erstmal fr klare Verhltnisse sorgen.....
   if ( (speicherstatus!=offen) && (speicherstatus!=geschlossen) ) // Im EEPROM steht noch scheisse.......
     auf ();
   if (status==offen) auf (); else zu ();
 } else
 
 { // Stimmt der eindeutige uere Zustand auch mit dem zuletzt gespeichertem berein ?
   if (status!=speicherstatus) 
    {  // Speichr aktualisieren
     speicherstatus=status;
     write_eeprom (statusadresse,speicherstatus);
    } 
 }

// Nu is ein mglicher Fehlerzutand behoben und alles ist beim alten....
       
 status=get_zustand ();			// Neuen Status reinholen

while (1)
      {
      #asm("wdr");
        
       altstatus=status;			// Alten Zustand ablegen
         
       status=get_zustand ();			// Neuen Status reinholen
  
       if (altstatus!=status) 			// Eine nderung ist eingetreten
        {
	  sdelay ();				// Ganz kurze Pause      
	  status=get_zustand ();        	// Neuen Status noch einmal reinholen
        }                                                                       
        
        if (altstatus!=status) 			// Eine nderung ist tatschlich eingetreten
         {					// Reaktion ist gefragt !
           if (altstatus==offen) zu (); else auf ();
           
        // Ist der Zustand jetzt eindeutig ? Wenn nicht -> Warten, bis jemand seine Griffel in der Mechanik versenkt hat...
        status=get_zustand ();			// Neuen Status reinholen
        while (status==unbekannt)     
        {         
          #asm("wdr");
          status=get_zustand ();			// Neuen Status reinholen
        }   

         }
       
      };
}
                 